 
c=============================================================================
c
c $Id: qj_update.F,v 1.2 2002/02/11 16:32:07 akinorii Exp $
c
c CODE DEVELOPER
c   John Tannahill, LLNL
c   jrt@llnl.gov
c
c   IMPACT quadchem modifications:  Akinori Ito.
c
c FILE
c   qj_update.F
c
c ROUTINES
c   Update_Qj
c
c=============================================================================
 
 
c-----------------------------------------------------------------------------
c
c ROUTINE
c   Update_Qj
c
c DESCRIPTION
c   This routine updates the photolysis rate constants (i.e., qjs).
c
c ARGUMENTS
c   imp_data_dir     : directory where input data files are located
c   metdata_name_org : first  part of metdata_name, e.g., "NCAR"
c   pr_qj_o3_o1d     : should special reaction O3->O1d be saved?
c   pr_qj_opt_depth  : should optical depth be saved at the end of qj file?
c   photintv  : photolysis time step  (s)
c   rsec_jan1 : seconds from Jan. 1st (s)
c   mass3     : total mass of the atmosphere within   each grid box (kg)
c   pres3c    : atmospheric pressure at the center of each grid box (mb)
c   temp3     : temperature (degK)
c   const     : species concentration, known at zone centers (mixing ratio)
c   latdeg    : latitude    (deg)
c   mcor      : area of grid box (m^2)
c   max_cloud : maximum overlap cloud fraction for LW
c   ran_cloud : random  overlap cloud fraction for LW
c   qjimp     : photolysis rate constants (s^-1)
c   ai          : pressure = (ai * pt) + (bi * psf), ai at zone interface
c   bi          : pressure = (ai * pt) + (bi * psf), bi at zone interface
c   pctm2       : psf, also CTM surface pressure at t1+tdt (mb)
c   humidity    : specific humidity (g/kg)
c   mass        : total mass of the atmosphere within each grid box   (kg)
c   cmf1        : convective mass flux at t1 (kg/m2*s)
c   grid_height : height of each grid box (m)
c   lwi_flags   : array of flags that indicate land, water, or ice
c   do_trop_chem: do tropospheric chemistry
c   londeg           : longitude (deg)
c   surf_alb_uv      : bulk surface albedo (fraction 0-1)
c   tau_cloud        : optical depth (dimensionless)
c
c-----------------------------------------------------------------------------

      subroutine Update_Qj
     &  ( metdata_name_org, pr_qj_o3_o1d, pr_qj_opt_depth,
     &   photintv, rsec_jan1,  pres3c, temp3,
     &   const, latdeg, mcor,  qjimp,
     &   ai, bi, pctm2,
     &   humidity, mass,
     &   cmf1, grid_height, lwi_flags, do_trop_chem,
     &   londeg, surf_alb_uv, tau_cloud,aero2d_opt, aero2d_ssa)
! delete imp_data_dir, max_cloud, ran_cloud, one redudant mass
! add aero2d_opt, aero2d_ssa 
       use h_imp_time_constants
       use h_imp_dims
       use h_imp_control
       use h_phot_lookup
       use h_imp_phys_constants
       use h_rbchemmech

       use abortutils,    only: endrun

       implicit none

c#     include "imp_time_constants.h"
c#     include "imp_dims.h"
c#     include "imp_control.h"
c#     include "phot_lookup.h"
c#     include "imp_phys_constants.h"

c Added by Sandy Sillman, 2007 (to pass chem. mechanism to rate calc.)
c#     include "rbchemmech.h"


c     ----------------------
c     Argument declarations.
c     ----------------------

c      character (len=80) :: imp_data_dir
      character (len=16) :: metdata_name_org

      logical :: do_trop_chem(i1:i2, ju1:j2, k1:k2)

      logical :: pr_qj_o3_o1d
      logical :: pr_qj_opt_depth

      integer :: lwi_flags(i1_gl:i2_gl, ju1_gl:j2_gl)

      real*8  :: photintv
      real*8  :: rsec_jan1
c      real*8  :: mass3      (i1:i2, ju1:j2, k1:k2)
      real*8  :: pres3c     (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: temp3      (ilo:ihi, julo:jhi, k1:k2)
      real*8  :: const      (i1:i2, ju1:j2, k1:k2, num_species)
      real*8  :: latdeg     (ju1_gl:j2_gl)
      real*8  :: mcor       (i1:i2, ju1:j2)
c      real*8  :: max_cloud  (i1:i2, ju1:j2, k1:k2)
c      real*8  :: ran_cloud  (i1:i2, ju1:j2, k1:k2)
      real*8  :: qjimp      (i1:i2, ju1:j2, k1:k2, num_qjs)
      real*8  :: ai         (k1-1:k2)
      real*8  :: bi         (k1-1:k2)
      real*8  :: pctm2      (ilo:ihi, julo:jhi)
      real*8  :: humidity   (i1:i2,   ju1:j2,   k1:k2)
      real*8  :: mass       (i1:i2, ju1:j2, k1:k2)
      real*8  :: cmf1       (i1:i2, ju1:j2, k1:k2)
      real*8  :: grid_height(i1:i2, ju1:j2, k1:k2)

!      real*8  :: londeg     (i1_gl:i2_gl)
      real*8  :: londeg     (ju1_gl:j2_gl) !Guangxing Lin
      real*8  :: surf_alb_uv(i1:i2, ju1:j2)
      real*8  :: tau_cloud  (i1:i2, ju1:j2, k1:k2)
      real*8  :: aero2d_opt  (i1:i2, ju1:j2)
      real*8  :: aero2d_ssa  (i1:i2, ju1:j2)

c     ----------------------
c     Function declarations.
c     ----------------------

      integer, external :: Convert_Nhms_Nsec


c     ----------------------
c     Variable declarations.
c     ----------------------

      integer :: idumday, idumyear
      integer :: il, ij, ik
      integer :: jday
      integer :: month_imp

      real*8  :: time_sec

      real*8  :: kel_ij     (k1:k2)
      real*8  :: optdepth_ij(k1:k2)

      real*8  :: qjimp_ij   (k1:chem_mask_khi, num_qjs)

      real*8  :: n2adj(i1:i2, ju1:j2, k1:k2)
      real*8  :: o2adj(i1:i2, ju1:j2, k1:k2)
      real*8  :: tauro(i1:i2, ju1:j2, k1:k2)

      real*8, parameter ::
     &  R_61  = (1.0d0 - (MWTH2O / MWTAIR)) / (MWTH2O / MWTAIR)

      real*8, parameter ::
     &  R29_3 = (GAS_CONST_J / MWTAIR * GPKG) / IMP_G

      real*8, parameter ::
     &  lwd = 1.0d+06,  ! liquid water densitry (g/m3)
     &  ext_eff = 2.0d0 ! extinction efficiency of a water droplet

c     cloud liquid water content in each grid box, g/m3
      real*8, allocatable :: lwccol(:,:,:)

c     cloud drop effective radius (m)
      real*8, allocatable :: eff_rad(:,:,:)

c     cloud fraction (unitless)
      real*8, allocatable :: cldfra(:,:,:)

c     height above sea surface (m) at grid box edge
      real*8, allocatable :: zlev_asl(:,:,:)

       integer :: ierror(100),ierr_count

       ierr_count=0
             ierr_count=ierr_count+1
      Allocate (lwccol(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
             ierr_count=ierr_count+1
      Allocate (eff_rad(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
             ierr_count=ierr_count+1
      Allocate (cldfra(i1:i2, ju1:j2, k1:k2),stat=ierror(ierr_count))
             ierr_count=ierr_count+1
      Allocate (zlev_asl(i1:i2, ju1:j2, k1-1:k2),stat=ierror(ierr_count))

        if ( any( ierror(1:ierr_count) .ne. 0 ) ) then
           print*, ierror(1:ierr_count)
           call endrun("Error in allocation in qj_Update")

         end if

      lwccol = 0.0d0
      eff_rad = 0.0d0
      cldfra = 0.0d0
      zlev_asl = 0.0d0

c     ----------------
c     Begin execution.
c     ----------------
 
      if (pr_diag) then
        Write (6,*) 'Update_Qj called by ', loc_proc
      end if


c       phot_opt =6 !set in h_init.F
c     ================== 
c      if (phot_opt == 3) then
c     ==================



 
 
c     ===========================
c      elseif (phot_opt == 6) then
c     ===========================
c       write(6,*)' testtest tuv_lookup'
c       ===============
        call Tuv_Lookup
c       ===============
     & (photintv, latdeg, rsec_jan1, 
     &  pres3c, pctm2, ai, bi, temp3, const,
     &  humidity, mcor, mass,
     &  cmf1, surf_alb_uv, grid_height, lwi_flags,
     &  qjimp, do_trop_chem, aero2d_opt, aero2d_ssa)

c      end if
 
c     ----------------------------------------------------------------------
c     Check to see if the mechanism has a photolysis reaction O3 + hv = 2OH.
c     If so, save the rate in the last entry of qjimp and then the real rate
c     needs to be adjusted.
c     ----------------------------------------------------------------------

!Guangxing Lin: not sure num_qj_o3_to_2oh is euqal to 0, but set it to 0 now!
!need to check it later!!
      num_qj_o3_to_2oh =0      
      if (num_qj_o3_to_2oh > 0) then

        n2adj(:,:,:) =
     &   1.8d-11 * Exp (110.0d0 / temp3(i1:i2,ju1:j2,:)) * MXRN2
        o2adj(:,:,:) = 
     &   3.2d-11 * Exp ( 70.0d0 / temp3(i1:i2,ju1:j2,:)) * MXRO2

        if (pr_qj_o3_o1d) then
          qjimp(:,:,:,num_qjs+1) = qjimp(:,:,:,num_qj_o3_to_2oh)
        end if

        qjimp(:,:,:,num_qj_o3_to_2oh) = 
     &    qjimp(:,:,:,num_qj_o3_to_2oh) /
     &    (1.0d0 +
     &     ((n2adj(:,:,:) + o2adj(:,:,:)) /
     &      (2.2d-10 * const(i1:i2,ju1:j2,:,ih2o_num))))

      end if

      Deallocate (eff_rad)
      Deallocate (lwccol)
      Deallocate (cldfra)
      Deallocate (zlev_asl)

      return

      end

